#!/bin/sh
# the next line restarts using tclsh \
exec tclsh "$0" "$@"

#***************************************************************************
# Licensed Materials - Property of IBM                                     *
#                                                                          *
# 5724-F72                                                                 *
#                                                                          *
# (C) Copyright IBM Corp. 1997, 2003  All Rights Reserved.                 *
#                                                                          *
# US Government Users Restricted Rights - Use, duplication or disclosure   *
# restricted by GSA ADP Schedule Contract with IBM Corp.                   *
#***************************************************************************

proc Exit {{rc 0}} {
    close stdout
    close stderr
    exit $rc
}

# set platform/OS specific globals
set VVTHOME VVT_HOME
switch -exact -- $tcl_platform(platform) {
  windows {
    package require registry 1.0
  }
}

# verify that $VVTHOME is set
if {[info exists ::env($VVTHOME)] == 0} {
  puts stderr "ERROR: Environment variable $VVTHOME is not set."
    switch -exact -- $tcl_platform(platform) {
    unix {
      switch -exact -- $tcl_platform(os) {
        AIX {
          puts stderr "Does the .profile for this user contain the following line:\n'. /var/vvt/bin/vvt.setenv'?"
        }
        Linux {
          puts stderr "Does the login profile for this user contain the following line:\n'. /opt/IBM/vvt/setenv'?"
        }
      }
      puts stderr "Does this user belong to group vvt?"
    }
  }
  Exit 1
}
# change to $VVTHOME
#cd $::env($VVTHOME)

# compose the filename to verify if copyright has been displayed already
set cr(fn) ".cr.$tcl_platform(user).vvt_config"
set cr(path) [file join $::env($VVTHOME) tmp $cr(fn)]

# show copyright if has not been shown already
if {[file exists $cr(path)] == 0} {
  puts stdout "*************************************************************************\n\n Licensed Materials - Property of IBM\n\n 5724-F72\n\n (C) Copyright IBM Corp. 1997, 2003  All Rights Reserved.\n\n*************************************************************************\n"
    if {[catch {open $cr(path) w+} fd] == 0} {
      catch {close $fd} hlp
    }
} else {
  puts stdout ""
}

# set platform/OS specific globals
switch -exact -- $tcl_platform(platform) {
  unix {
    switch -exact -- $tcl_platform(os) {
      AIX {
# AIX installp package names
        set package(Asr)            viavoice.asr.
        set package(Client)         viavoice.client.rte
        set package(Server)         viavoice.server.com
        set package(Server.Asr)     viavoice.asr_server.rte
        set package(Server.Tts)     viavoice.tts_server.rte
        set package(Tts)            [list viavoice.tts. ibmtts.]
        set package(ScanSoft)       [list realspeak.engine_]
      }
      Linux {
# Linux RPM package names
        set package(Asr)            viavoice
        set package(Client)         vvtclient
        set package(Server)         vvtserver
        set package(Server.Asr)     vvtserver
        set package(Server.Tts)     vvtserver
        set package(Tts)            ibmtts_rte
        set package(ScanSoft)       realspeak
      }
    }
  }
  windows {
# Windows Registry Key/Subdirectory
    set package(Asr)            [ file join $env($VVTHOME) ViaVoice vocabs langs ]
    set package(Client)         "HKEY_LOCAL_MACHINE\\SOFTWARE\\IBM\\Voice Server\\VoiceResponse"
    set package(Server)         $package(Client)
    set package(Server.Asr)     $package(Asr)
    set package(Server.Tts)     "HKEY_LOCAL_MACHINE\\SOFTWARE\\IBM\\ViaVoice TTS"
    set package(Tts)            "$package(Server.Tts)"
    set package(ScanSoft)       {}
  }
}
# the following line is always identical with Tts!
set package(Tts.Multi)          "$package(Tts)"

# set platform/OS independend globals
set config(script)          [lindex [file split $argv0] end]
set config(ROOT)            [file join $env($VVTHOME) cfg]
set config(fn,VVTdefaults)  VVTdefaults
set config(fn,procman.cfg)  procman.cfg
set config(fn,.defaults)    .defaults
set config(read_defaults)   1
set config(autoresponse)    0

set filename(VVTdefaults)   [file join $config(ROOT) $config(fn,VVTdefaults)]
set filename(procman.cfg)   [file join $config(ROOT) $config(fn,procman.cfg)]
set filename(.defaults)     [file join $config(ROOT) $config(fn,.defaults)]

# verify that cfg subdirectory exists
if {[file exists $config(ROOT)] == 0} {
  puts stderr "ERROR: configuration directory $config(ROOT) does not exist."
    Exit 1
}
# start with an empty configuration
set config(VVTdefaults)     {}
# strings used for preserving user modified data in the configuration
set config(COMMON)          {# DO NOT MODIFY THIS LINE - THE CONFIGURATION UTILITY DEPENDS ON IT - %s}
set config(WARNING)         "#\n# EVERY MODIFICATION BETWEEN THE TWO 'DO NOT MODIFY' LINES WILL BE LOST\n# DURING RECONFIGURATION\n#"
set config(BEGIN)           [format $config(COMMON) "BEGIN"]
set config(END)             [format $config(COMMON) "END"]

# which services should be started by default
set configure(logger)       1
set configure(router)       0
set configure(Asr)          0
set configure(Tts)          0

# set specific speech recognition values
# 1. a list of available tasks, for each task a 2 item list with name and desc
# 2. UID
# 3. EID
# 4. baseformSource
#set settings(Asr,sample) {{{vebAB "Task AB"} {vebCD "Task CB"}} vsab_cd abcd vv}
set settings(Asr,da_DK) {{{vebda "ViaVoice for E-Business"}} vsda_dk vxml vv}
set settings(Asr,de_DE) {{{vebgr "ViaVoice for E-Business"}} vsde_de vxml vv}
set settings(Asr,en_AU) {{{vebau "ViaVoice for E-Business"}} vsen_au vxml vv}
set settings(Asr,en_GB) {{{vebuk "ViaVoice for E-Business"}} vsen_gb vxml vv}
set settings(Asr,en_US) {{{vebus "ViaVoice for E-Business"}} vsen_us vse1 vv}
set settings(Asr,es_ES) {{{vebes "ViaVoice for E-Business"}} vses_es vxml vv}
set settings(Asr,fr_CA) {{{vebca "ViaVoice for E-Business"}} vsfr_ca vxml vv}
set settings(Asr,fr_FR) {{{vebfr "ViaVoice for E-Business"}} vsfr_fr vxml vv}
set settings(Asr,it_IT) {{{vebit "ViaVoice for E-Business"}} vsit_it vxml vv}
set settings(Asr,ja_JP) {{{vebjp "ViaVoice for E-Business"}} vsja_jp vsj1 vv}
set settings(Asr,ko_KR) {{{vebkr "ViaVoice for E-Business"}} vsko_kr vxml vv}
set settings(Asr,nl_NL) {{{vebnl "ViaVoice for E-Business"}} vsnl_nl vxml vv}
set settings(Asr,no_NO) {{{vebno "ViaVoice for E-Business"}} vsno_no vxml vv}
set settings(Asr,pt_BR) {{{vebbr "ViaVoice for E-Business"}} vspt_br vxml vv}
set settings(Asr,sv_SE) {{{vebsv "ViaVoice for E-Business"}} vssv_se vxml vv}
set settings(Asr,zh_CN) {{{vebcn "ViaVoice for E-Business"}} vszh_cn vsz1 vv}
set settings(Asr,zh_HK) {{{vebhk "ViaVoice for E-Business"}} vszh_hk vxml vv}
set settings(Asr,zh_TW) {{{vebtw "ViaVoice for E-Business"}} vszh_tw vxml vv}

# set some maximum values
set max(channels)           9998
set max(retry)              5
set max(charperline)        78

# ID for output written by script
proc get_id {} {
  return {# $Source: /home/cvsrepo/vvt/bin/tools/vvtconfig.tcl,v $ $Revision: 1.15.2.7.2.6.2.15.2.9 $ $Date: 2003/09/23 14:01:09 $}
}

# timestamp for output written by script
proc get_timestamp {} {
  global tcl_platform
    
  return "# created on [clock format [clock seconds] -format "%Y/%m/%d %H:%M:%S"] by $tcl_platform(user)"
}

# determine which languages are installed for given type (Asr, Tts)
proc get_installed_languages {type} {
  global tcl_platform cache

  if {[info exists cache(il,$type)] == 0} {
    set rc {}
    switch -exact -- $type {
      Asr -
      Tts -
      Tts.Multi -
      ScanSoft {
        # check and call the platform specific procedure
        if {[info commands "$tcl_platform(platform)_get_installed_languages"] != ""} {
          set rc [$tcl_platform(platform)_get_installed_languages $type]
        } else {
          puts stderr "ERROR: unsupported platform: $tcl_platform(platform)"
        }
      }
      default {
        puts stderr "ERROR: unsupported service: $type"
      }
    }
    # convert all language codes to xx_XX format
    for {set i 0} {$i < [llength $rc]} {incr i} {
      regsub -all -- {-} [string tolower [lindex $rc $i] 0 1] {_} item
      set rc [lreplace $rc $i $i $item]
    }
    # workaround for wrong ASR/TTS language codes
    regsub -all -nocase -- en_UK $rc {en_GB} rc
    regsub -all -nocase -- gr_GR $rc {de_DE} rc
    # workaround for ScanSoft language codes
    regsub -all -nocase -- dun $rc {nl_NL} rc
    regsub -all -nocase -- iti $rc {it_IT} rc
    regsub -all -nocase -- cah $rc {zh_HK} rc
    switch -exact -- $type {
      Tts.Multi {
        # do not display unsupported languages for multi-language TTS
        regsub -all -- {ja_[A-Z][A-Z]} $rc {} rc
        regsub -all -- {ko_[A-Z][A-Z]} $rc {} rc
        regsub -all -- {zh_[A-Z][A-Z]} $rc {} rc
        # do not display ScanSoft languages
        regsub -all -- {nl_NL} $rc {} rc
        regsub -all -- {it_IT} $rc {} rc
        regsub -all -- {zh_HK} $rc {} rc
      }
    }
    set cache(il,$type) [lsort -unique $rc]
  }
  return $cache(il,$type)
}

# determine which languages are install on Unix
proc unix_get_installed_languages {type} {
  global tcl_platform package
    
  set langs {}
  switch -exact -- $tcl_platform(os) {
    AIX {
      foreach root $package($type) {
        if {[catch {open "|lslpp -l ${root}* b${root}*" RDWR} fd] == 0} {
          while {[eof $fd] != 1} {
            gets $fd line
            if {[regexp -- "${root}\[a-zA-Z\]*\[-_\]\[a-zA-Z\]*" $line] == 1 ||
                [regexp -- "$package(ScanSoft)\[a-zA-Z\]\[a-zA-Z\]\[a-zA-Z\]" $line] == 1} {
              regsub -- "^.*${root}" $line {} line
              regsub -- {[ \.].*$} $line {} line
              lappend langs $line
            }
          }
          catch {close $fd} hlp
        } else {
          puts stderr "ERROR: during quering installed languages. ($fd)"
          Exit 1
        }
      }
    }
    Linux {
      if {[catch {open "|rpm -q -a" RDWR} fd] == 0} {
        while {[eof $fd] != 1} {
          gets $fd line
          foreach root $package($type) {
            if {[regexp -- "^${root}_\[a-zA-Z\]*\[-_\]\[a-zA-Z\]*" $line] == 1} {
              regsub -- "^${root}." $line {} line
              regsub -- {-[0-9\.-]$} $line {} line
              if {[string length $line] >= 5} {
                lappend langs [string range $line 0 4]
              }
            }
          }
        }
        catch {close $fd} hlp
      } else {
        puts stderr "ERROR: during quering installed languages. ($fd)"
        Exit 1
      }
    }
    default {
      puts stderr "ERROR: unsupported OS: $tcl_platform(os)"
    }
  }
  return $langs
}

# determine which languages are install on Windows
proc windows_get_installed_languages {type} {
  global package
    
  set langs {}
  switch -exact -- $type {
    Asr {
      if {[file exists $package(Asr)] && [file isdirectory $package(Asr)]} {
        set cwd [pwd]
        cd $package(Asr)
        set langs [glob -nocomplain -type d *]
        cd $cwd
      }
    }
    Tts -
    Tts.Multi {
      set langs [registry keys $package(Tts)]
    }
    ScanSoft {
    }
    default {
      puts stderr "ERROR: unsupported service: $type"
    }
  }
  return $langs
}

# determine if this machine is a client/server and if server for which type
proc is {type} {
  global tcl_platform cache

  if {[info exists cache(is,$type)] == 0} {
    set rc -1
    switch -exact -- $type {
      Client -
      Server -
      Server.Asr -
      Server.Tts {
        if {[info commands "$tcl_platform(platform)_get_installed_languages"] != ""} {
          set rc [$tcl_platform(platform)_is $type]
        } else {
          puts stderr "ERROR: unsupported platform: $tcl_platform(platform)"
        }
      }
      default {
        puts stderr "ERROR: unsupported service: $type"
      }
    }
    set cache(is,$type) $rc
  }
  return $cache(is,$type)
}

# determine if this machine is a client/server and if server for which type under Unix
proc unix_is {type} {
  global tcl_platform package
    
  set is 0
  switch -exact -- $tcl_platform(os) {
    AIX {
      set root $package($type)
      if {[catch {open "|lslpp -l ${root} b${root}" RDWR} fd] == 0} {
        while {[eof $fd] != 1} {
          gets $fd line
          if {[regexp -- "${root}" $line] == 1} {
            set is 1
            break
          }
        }
        catch {close $fd} hlp
      } else {
        puts stderr "ERROR: during quering installed server packages. ($fd)"
        Exit 1
      }
    }
    Linux {
      set root $package($type)
      if {[catch {open "|rpm -q -a" RDWR} fd] == 0} {
        while {[eof $fd] != 1} {
          gets $fd line
          if {[regexp -- "^${root}-\[0-9\.-]*\$" $line] == 1} {
            set is 1
            break
          }
        }
        catch {close $fd} hlp
      } else {
        puts stderr "ERROR: during quering installed server packages. ($fd)"
        Exit 1
      }
    }
    default {
      puts stderr "ERROR: unsupported service: $type"
      set is -1
    }
  }
  return $is
}

# determine if this machine is a client/server and if server for which type under Windows
proc windows_is {type} {
  global package
    
  set is_asr -1
  set is_tts -1
  set is -1
    
  if {[file exists $package(Asr)] && [file isdirectory $package(Asr)]} {
    set is_asr 1
  }
  if {[catch {registry keys $package(Server.Tts)} rc] == 0} {
    set is_tts 1
  }
  switch -exact -- $type {
    Client {
      if {[catch {registry get $package($type) SetupType} rc] == 0 &&
          ([string compare -nocase $rc {client}] == 0 ||
           [string compare -nocase $rc {server}] == 0)} {
        set is 1
      }
    }
    Server {
      if {$is_asr == 1 || $is_tts == 1} {
        set is 1
      }
    }
    Server.Asr {
      set is $is_asr
    }
    Server.Tts {
      set is $is_tts
    }
    default {
      puts stderr "ERROR: unsupported service: $type"
      set is -1
    }
  }
  return $is
}

proc query_print {question {range {}} {default {}}} {
  global max
    
  if {[string compare $default {}] != 0} {
    if {[string compare $range {}] != 0} {
      set question "${question}_($range)?_\[_${default}_]"
    } else {
      set question "$question?_\[_${default}_]"
    }
  } else {
    if {[string compare $range {}] != 0} {
      set question "${question}_($range)?"
    } else {
      set question "$question?"
    }
  }
  while {[string length $question] > 0} {
    set break $max(charperline)
    if {[string length $question] >= $max(charperline)} {
      # search from the last character back for a breakable position
      while {[string compare [string trim [string index $question $break]] {}] != 0 && $break >= 0} {
        incr break -1
      }
      # if we did not find a breakable position search for the first possible
      if {$break == -1} {
        set break $max(charperline)
        while {[string compare [string trim [string index $question $break]] {}] != 0 && $break >= 0} {
          incr break
        }
      }
      set out [string trim [string range $question 0 $break]]
    } else {
      set out $question
    }
    regsub -- {_\[_} $out { [ } out
    regsub -- {_]} $out { ]} out
    regsub -- {_\(} $out { (} out
    puts stdout $out
    set question [string trim [string range $question $break end]]
  }
}

proc query_path {question default} {
  global max config
    
  set rc -1
  set valid 0
  incr valid -$max(retry)
  while {$valid < 1} {
    if {$valid == 0} {
      puts stderr "You gave $max(retry) invalid answers. Terminating."
      Exit 1
    }
    incr valid
    if {$config(autoresponse) == 0} {
      query_print $question {} $default
      set length [gets stdin answer]
    } else {
      set length 0
    }
    if {$length == 0} {
      set answer $default
    }
    if {[file exists $answer] == 1} {
      if {[file isdirectory $answer] == 1} {
        set rc $answer
        set valid 1
      } else {
        puts stderr "You did not specify a valid directory."
      }
    } else {
      puts stderr "The directory does not exist or you do not have sufficient permissions."
    }
  }
  return $rc
}

proc query_engine_type {question default} {
  global max config
    
  set rc -1
  set valid 0
  incr valid -$max(retry)
  while {$valid < 1} {
    if {$valid == 0} {
      puts stderr "You gave $max(retry) invalid answers. Terminating."
      Exit 1
    }
    incr valid
    if {$config(autoresponse) == 0} {
      query_print $question {} $default
      set length [gets stdin answer]
    } else {
      set length 0
    }
    if {$length == 0} {
      set answer $default
    }
    if {[regexp -- {[[:space:]]} $answer] == 0} {
      set rc $answer
      set valid 1
    } else {
      puts stderr "The engine type contains white space character(s)."
    }
  }
  return $rc
}

proc query_hostname {question default {minimum 1} {maximum -1}} {
  global max config
    
  set host {}
  set valid 0
  incr valid -$max(retry)
  while {$valid < 1} {
    if {$valid == 0} {
      puts stderr "You gave $max(retry) invalid answers. Terminating."
      Exit 1
    }
    incr valid
    if {$config(autoresponse) == 0} {
      query_print $question {} $default
      set length [gets stdin answer]
    } else {
      set length 0
    }
    if {$maximum != -1 && [llength $answer] > $maximum} {
      puts stderr "Only one hostname allowed."
    } else {
      if {$length == 0} {
        set answer $default
        set length 1
      }
      # remove commas from provided input
      regsub -all -- {,} $answer { } answer
      if {$length > 0} {
        set ok 0
        foreach h $answer {
          catch {socket $h 7} s
          if {[string match {sock[0-9]*} $s] == 1} {
            incr ok
            catch {close $s} hlp
          } elseif {[regexp -- {connection refused} $s] == 1} {
            incr ok
          } else {
            puts stderr "$h: $s"
          }
        }
        if {$ok < $minimum} {
          puts stderr "At least $minimum hostname is required."
        } else {
          if {[llength $answer] == $ok} {
            set host $answer
            set valid 1
          }
        }
      }
    }
  }
  return $host
}

proc query_number {question default minimum maximum} {
  global max config
    
  set rc -1
  if {$minimum > $maximum} {
    set hlp $minimum
    set minimum $maximum
    set maximum $hlp
  }
  if {$default < $minimum} {
    set default $minimum
  }
  if {$default > $maximum} {
    set default $maximum
  }
  if {$minimum == $maximum} {
    return $rc
  }
  set valid 0
  incr valid -$max(retry)
  while {$valid < 1} {
    if {$valid == 0} {
      puts stderr "You gave $max(retry) invalid answers. Terminating."
      Exit 1
    }
    incr valid
    if {$config(autoresponse) == 0} {
      query_print $question "$minimum-$maximum" $default
      set length [gets stdin answer]
    } else {
      set length 0
    }
    if {$length == 0} {
      set answer $default
    }
    if {[regexp -- {^[[:digit:]]+$} $answer] == 1 &&
        $answer >= $minimum && $answer <= $maximum} {
      set rc $answer
      set valid 1
    }
  }
  return $rc
}

proc query_yesno {question default} {
  global max config
    
  set rc -1
  set d {}
  switch -exact -- $default {
    1 {
      set d "Y"
    }
    0 {
      set d "N"
    }
    default {
      puts stderr "INTERNAL: unsupported default: $default"
      Exit 2
    }
  }
  set valid 0
  incr valid -$max(retry)
  while {$valid < 1} {
    if {$valid == 0} {
      puts stderr "You gave $max(retry) invalid answers. Terminating."
      Exit 1
    }
    incr valid
    if {$config(autoresponse) == 0} {
      query_print $question "y/n" $d
      set length [gets stdin answer]
    } else {
      set length 0
    }
    if {$length == 0} {
      set answer $d
    }
    switch -exact -- $answer {
      y -
      Y {
        set rc 1
        set valid 1
      }
      n -
      N {
        set rc 0
        set valid 1
      }
      default {
      }
    }
  }
  return $rc
}

proc translate_language_code {rule code} {
  set out {}
  set delimiter { }
  switch -exact -- $rule {
    STDVVT {
      foreach c [split $code { }] {
        switch -regexp -- $c {
          [[:lower:]][[:lower:]]\_[[:upper:]][[:upper:]] -
          [[:upper:]][[:lower:]]\_[[:upper:]][[:upper:]] {
            regsub -all -- {_} [string toupper $c] {-} t
          }
          [[:upper:]][[:upper:]]\-[[:upper:]][[:upper:]] {
            regsub -all -- {-} [string tolower $c 0 1] {_} t
          }
          default {
            set t $c
            puts stderr "ERROR: Language code: $c does not match any rule."
          }
        }
        append out "${delimiter}$t"
      }
    }
    STRING {
      set delimiter {, }
      foreach c $code {
        switch -regexp -- $c {
          [dD][aA][_-]DK { set t "Danish" }
          [dD][eE][_-]DE { set t "German" }
          [eE][nN][_-]AU { set t "Australian English" }
          [eE][nN][_-]GB { set t "U.K. English" }
          [eE][nN][_-]US { set t "U.S. English" }
          [eE][sS][_-]ES { set t "Spanish" }
          [eE][sS][_-]MX { set t "Mexican Spanish" }
          [fF][iI][_-]FI { set t "Finnish" }
          [fF][rR][_-]CA { set t "Canadian French" }
          [fF][rR][_-]FR { set t "French" }
          [iI][tT][_-]IT { set t "Italian" }
          [jJ][aA][_-]JP { set t "Japanese" }
          [kK][oO][_-]KR { set t "Korean" }
          [nN][lL][_-]NL { set t "Dutch" }
          [nN][oO][_-]NO { set t "Norwegian" }
          [pP][tT][_-]BR { set t "Brazilian Portuguese" }
          [sS][vV][_-]SE { set t "Swedish" }
          [zZ][hH][_-]CN { set t "Simplified Chinese" }
          [zZ][hH][_-]HK { set t "Cantonese" }
          [zZ][hH][_-]TW { set t "Traditional Chinese" }
          default {
            set t $c
            puts stderr "Language code: $c unknown - no translation performed."
          }
        }
        append out "${delimiter}$t"
      }
    }
    default {
      set out $code
      puts stderr "Translation rule: ${rule} unknown - no translation performed."
    }
  }
  return [string range $out [string length $delimiter] end]
}

proc read_user_modified_data {_cfg} {
  global env config
    
  set cfg [file join $config(ROOT) $_cfg]
  if {[file exists $cfg] == 1 && [file size $cfg] > 0} {
    puts -nonewline stdout "Integrating user modified data from current $_cfg "
    if {[catch {open $cfg RDONLY} fd] == 0} {
      set n 0
      set skip -1
      while {[gets $fd line] != -1} {
        if {[string compare $line $config(BEGIN)] == 0} {
          set skip 0
        }
        if {$skip != 0} {
          if {$skip == -1} {
            set config($_cfg) [linsert $config($_cfg) $n $line]
            incr n
          }
          if {$skip == 1} {
            lappend config($_cfg) $line
          }
          puts -nonewline stdout "."
        }
        if {[string compare $line $config(END)] == 0} {
          set skip 1
        }
      }
      catch {close $fd} hlp
      puts stdout {}
      if {$skip == -1} {
        puts stdout "Current $_cfg does not contain the following marker:\n\n$config(BEGIN)\n\nNo user specific data copied."
        incr n -1
        set config($_cfg) [lreplace $config($_cfg) 0 $n]
      }
    } else {
      puts stderr "WARNING: could not open $cfg for reading. ($fd)"
    }
  }
}

proc read_defaults {} {
  global env config default filename

  if {$config(read_defaults) == 1} {
    if {[file exists $filename(.defaults)] == 1 && [file readable $filename(.defaults)] == 1} {
      catch {source $filename(.defaults)} hlp
    }
  }
}

proc write_defaults {} {
  global env config default filename
  
  if {$config(autoresponse) == 0} { 
    if {[file exists $filename(.defaults)] == 0 || [file writable $filename(.defaults)] == 1} {
      if {[catch {open $filename(.defaults) [list WRONLY CREAT TRUNC]} fd] == 0} {
        puts $fd [get_id]
        puts $fd [get_timestamp]
        puts $fd {}
        puts $fd "array set default [list [array get default]]"
        catch {close $fd} hlp
      } else {
        puts stderr "WARNING: could not open $filename(.defaults) for writing. ($fd)"
      }
    }
  }
}

proc write_config {} {
  global env config filename
    
  set config(VVTdefaults) [linsert $config(VVTdefaults) 0 $config(WARNING)]
  set config(VVTdefaults) [linsert $config(VVTdefaults) 0 [get_id]]
  set config(VVTdefaults) [linsert $config(VVTdefaults) 0 [get_timestamp]]
  set config(VVTdefaults) [linsert $config(VVTdefaults) 0 $config(BEGIN)]
  lappend config(VVTdefaults) $config(END)
  read_user_modified_data VVTdefaults
    
  if {([file exists $filename(VVTdefaults)] == 1 && [file size $filename(VVTdefaults)] > 0) || ([file exists $filename(procman.cfg)] == 1 && [file size $filename(procman.cfg)] > 0)} {
    set i 1
    while {[file exists [set filename(VVTdefaults.old) "$filename(VVTdefaults).[format "%03d" $i]"]] == 1 ||
           [file exists [set filename(procman.cfg.old) "$filename(procman.cfg).[format "%03d" $i]"]] == 1} {
      incr i
    }
    if {[file exists $filename(VVTdefaults)] == 1} {
      file rename -force -- $filename(VVTdefaults) $filename(VVTdefaults.old)
      puts stdout "Renamed $filename(VVTdefaults) to $filename(VVTdefaults.old)"
    }
    if {[file exists $filename(procman.cfg)] == 1} {
      file rename -force -- $filename(procman.cfg) $filename(procman.cfg.old)
      puts stdout "Renamed $filename(procman.cfg) to $filename(procman.cfg.old)"
    }
  }
    
  puts -nonewline stdout "Writing new configuration "
  if {[catch {open $filename(VVTdefaults) [list WRONLY CREAT TRUNC]} fd] == 0} {
    set i 0
    while {$i < [llength $config(VVTdefaults)]} {
      puts $fd [lindex $config(VVTdefaults) $i]
      puts -nonewline stdout "."
      incr i
    }
    catch {close $fd} hlp
  } else {
    puts stderr "ERROR: could not open $filename(VVTdefaults) for writing. ($fd)"
    Exit 1
  }
  puts stdout {}
}

proc parse_input {} {
  global argc argv argv0 config filename VVTHOME

  if {$argc != 0} {
    for {set i 0} {$i < $argc} {incr i} {
      switch -exact -- [lindex $argv $i] {
        --default -
        -d {
          set config(read_defaults) 0
        }
        --autoresponse -
        -a {
          set config(autoresponse) 1
        }
        --config -
        -c {
          incr i
          if {$i < $argc} {
            set temp [lindex $argv $i]
            if {[file exists $temp] == 1 && [file readable $temp] == 1} {
              set filename(.defaults) $temp
            } else {
              puts stderr "File $temp does not exist or is not readable - option ignored."
            }
          } else {
            puts stderr "Missing filename - option ignored."
          }
        }
        default {
          puts stderr "$config(script) is used to generate your configuration\nfiles under [file join $::env($VVTHOME) cfg].\n"
          puts stderr "USAGE:"
#          puts stderr "\t$config(script) \[-d] \[-c filename] \[-q]"
          puts stderr "\t$config(script) \[-d]"
          puts stderr "\nPARAMETERS:"
          puts stderr "\t<-?  | -h  |  --help>      displays this text."
          puts stderr "\t<-d  | --default>          use built-in default values.\n"
#          puts stderr "\t<-a  | --autoresponse>     use default values and do not ask questions.\n"
#          puts stderr "\t<-c  | --config> filename  read an alternative default values file.\n\t\t\t(default: $filename(.defaults))"
          puts stderr ""
          Exit 1
        }
      }
    }
  }
}

parse_input

read_defaults

if {[is Server] == 1} {
# the server package is installed
  set sc 0
  if {[is Server.Asr] == 1} {
# we have an ASR server
    set noprocesses 0
    while {[llength [get_installed_languages Asr]] > 0 &&
           $configure(Asr) == 0 &&
           $noprocesses < $max(retry)} {
      set channel 0
      foreach l [get_installed_languages Asr] {
        if {[info exists default(Asr,$l,processes)] == 0} {
          set default(Asr,$l,processes) 0
        }
        set n [set default(Asr,$l,processes) [query_number "How many [translate_language_code STRING $l] speech recognition engines do you want" $default(Asr,$l,processes) 0 [expr $max(channels) - $sc - $channel]]]
        if {$n > 0} {
          set tc 0
          set nt 0
          if {[string compare [array names settings "Asr,$l"] {}] == 0} {
            puts stderr "INTERNAL: missing speech recognition settings for $l"
            Exit 2
          }
          set tasks [lindex $settings(Asr,$l) 0]
          set uid [lindex $settings(Asr,$l) 1]
          set eid [lindex $settings(Asr,$l) 2]
          set bsf [lindex $settings(Asr,$l) 3]
          set tid {}
          set i 0
          while {$i < [llength $tasks] && $tc < $n} {
            if {[llength $tasks] > 1} {
              set j 0
              puts stdout {}
              while {$j < [llength $tasks]} {
                set k $j
                incr j
                puts stdout [format { %2d. %s} $j [lindex [lindex $tasks $k] 1]]
              }
              if {[info exists default(Asr,$l,$i,task)] == 0} {
                set default(Asr,$l,$i,task) 0
              }
              set t [set default(Asr,$l,$i,task) [query_number "Which task do you want to use" $default(Asr,$l,$i,task) 1 [llength $tasks]]]
              set task [lindex $tasks [expr $t - 1]]
              set tid [lindex $task 0]
              set desc [lindex $task 1]
              if {[info exists default(Asr,$l,$t,processes)] == 0} {
                set default(Asr,$l,$t,processes) [expr $n - $tc]
              }
              set nt [set default(Asr,$l,$t,processes) [query_number "What is the engine type" $default(Asr,$l,$t,processes) 0 [expr $n - $tc]]]
            } else {
              set t 1
              set nt $n
            }
            if {$nt > 0} {
              set task [lindex $tasks [expr $t - 1]]
              set tid [lindex $task 0]
              set desc [lindex $task 1]
              set question "What is the engine type"
              set dfl "asr$l"
              if {$t != 1} {
                append dfl $t
              }
              if {[info exists default(Asr,$l,$t,engine_type)] == 0} {
                set default(Asr,$l,$t,engine_type) $dfl
              }
              set engine_type [set default(Asr,$l,$t,engine_type) [query_engine_type $question $default(Asr,$l,$t,engine_type)]]
              set question "What is the application path"
              if {[info exists default(Asr,$l,$t,appPath)] == 0} {
                set default(Asr,$l,$t,appPath) $env($VVTHOME)
#                foreach path [list /nfs/VXMLGrammars $env($VVTHOME)] {
#                  if {[file exists $path] == 1 && [file isdirectory $path] == 1} {
#                    set default(Asr,$l,$t,appPath) $path
#                    break
#                  }
#                }
              }
              set appPath [set default(Asr,$l,$t,appPath) [query_path $question $default(Asr,$l,$t,appPath)]]
              set range $channel
              if {$nt > 1} {
                set range "${channel}-[expr $channel + $nt - 1]"
              }
              lappend config(VVTdefaults) "asrp.${range}.Asr.engineType: $engine_type"
              lappend config(VVTdefaults) "asrp.${range}.Asr.userId: $uid"
              lappend config(VVTdefaults) "asrp.${range}.Asr.enrollId: $eid"
              lappend config(VVTdefaults) "asrp.${range}.Asr.taskId: $tid"
              lappend config(VVTdefaults) "asrp.${range}.Asr.appPath: $appPath"
# as the lexer got removed the following line is not needed anymore
#              lappend config(VVTdefaults) "asrp.${range}.Tts.languages: [translate_language_code STDVVT $l]"
#              if {[string compare $bsf vv] != 0} {
              lappend config(VVTdefaults) "asrp.${range}.Asr.baseformsSource: $bsf"
#              }
              incr tc $nt
              incr channel $nt
            }
            incr i
          }
        }
      }
      if {$channel >= 1} {
        lappend config(VVTdefaults) "Pm.numberOfAsrServers: $channel"
        switch -exact -- $tcl_platform(platform) {
          unix {
            switch -exact -- $tcl_platform(os) {
              AIX {
                lappend config(VVTdefaults) "*.*.Asr.notifyWhenLoaded: 1"
              }
              Linux {
              }
            }
          }
          windows {
            lappend config(VVTdefaults) "*.*.Asr.notifyWhenLoaded: 1"
          }
        }
        set configure(Asr) 1
      }
      incr sc $channel
      if {$configure(Asr) == 0} {
        if {[info exists default(Asr,noprocesses)] == 0} {
          set default(Asr,noprocesses) 0
        }
        if {[set default(Asr,noprocesses) [query_yesno "You have defined no speech recognition engines, so you won't be able to use this server for speech recognition. Are you sure" $default(Asr,noprocesses)]] == 1} {
          set noprocesses $max(retry)
        } else {
          incr noprocesses
          if {$noprocesses >= $max(retry)} {
            puts stderr "You have reached the maximum number of retries for this question."
            puts stderr "Continuing without any defined speech recognition engines."
            puts stderr ""
          }
        }
      }
    }
  }
  if {[is Server.Tts] == 1} {
# we have a TTS server
    set noprocesses 0
    while {([llength [get_installed_languages Tts]] > 0 ||
            [llength [get_installed_languages ScanSoft]] > 0) &&
           $configure(Tts) == 0 &&
           $noprocesses < $max(retry)} {
      set channel 0
      set n 0
      if {[llength [get_installed_languages Tts.Multi]] > 1} {
        if {[info exists default(Tts,processes)] == 0} {
          set default(Tts,processes) 0
        }
        set n [set default(Tts,processes) [query_number "How many language-independent text-to-speech engines do you want ([translate_language_code STRING [get_installed_languages Tts.Multi]])" $default(Tts,processes) 0 [expr $max(channels) - $sc - $channel]]]
        if {$n > 0} {
          set question "What is the engine type"
          if {[info exists default(Tts,engine_type)] == 0} {
            set default(Tts,engine_type) "tts"
          }
          set engine_type [set default(Tts,engine_type) [query_engine_type $question $default(Tts,engine_type)]]
          set range $channel
          if {$n > 1} {
            append range "-[expr $channel + $n - 1]"
          }
          lappend config(VVTdefaults) "ttsp.${range}.Tts.engineType: $engine_type"
          lappend config(VVTdefaults) "ttsp.${range}.Tts.languages: [translate_language_code STDVVT [get_installed_languages Tts.Multi]]"
          incr channel $n
        }
      }
      if {[info exists default(Tts,single)] == 0} {
        set default(Tts,single) 0
      }
      if {$n == 0 ||
          [llength [get_installed_languages Tts]] != [llength [get_installed_languages Tts.Multi]] ||
          [llength [get_installed_languages ScanSoft]] > 0 ||
          [set default(Tts,single) [query_yesno "Do you want any single-language text-to-speech engines" $default(Tts,single)]] == 1} {
        foreach l [get_installed_languages Tts] {
          if {[info exists default(Tts,$l,processes)] == 0} {
            set default(Tts,$l,processes) 0
          }
          set n [set default(Tts,$l,processes) [query_number "How many [translate_language_code STRING $l] text-to-speech engines do you want" $default(Tts,$l,processes) 0 [expr $max(channels) - $sc - $channel]]]
          if {$n > 0} {
            set question "What is the engine type"
            if {[info exists default(Tts,$l,engine_type)] == 0} {
              set default(Tts,$l,engine_type) "tts$l"
            }
            set engine_type [set default(Tts,$l,engine_type) [query_engine_type $question $default(Tts,$l,engine_type)]]
            set range $channel
            if {$n > 1} {
              set range "${channel}-[expr $channel + $n - 1]"
            }
            lappend config(VVTdefaults) "ttsp.${range}.Tts.engineType: $engine_type"
            lappend config(VVTdefaults) "ttsp.${range}.Tts.languages: [translate_language_code STDVVT $l]"
            incr channel $n
          }
        }
        foreach l [get_installed_languages ScanSoft] {
          if {[info exists default(Tts,$l,processes)] == 0} {
            set default(Tts,$l,processes) 0
          }
          set n [set default(Tts,$l,processes) [query_number "How many [translate_language_code STRING $l] text-to-speech engines (ScanSoft) do you want" $default(Tts,$l,processes) 0 [expr $max(channels) - $sc - $channel]]]
          if {$n > 0} {
            set question "Should the audio encoding be alaw (No=mulaw)"
            if {[info exists default(ScanSoft,$l,encoding)] == 0} {
              set default(ScanSoft,$l,encoding) 1
            }
            set audio_encoding [set default(ScanSoft,$l,encoding) [query_yesno $question $default(ScanSoft,$l,encoding)]]
            set question "What is the engine type"
            if {[info exists default(ScanSoft,$l,$audio_encoding,engine_type)] == 0} {
              set default(ScanSoft,$l,$audio_encoding,engine_type) "SStts$l"
              if {$default(ScanSoft,$l,encoding) == 1} {
                append default(ScanSoft,$l,$audio_encoding,engine_type) "a"
              } else {
                append default(ScanSoft,$l,$audio_encoding,engine_type) "m"
              }
            }
            set engine_type [set default(ScanSoft,$l,$audio_encoding,engine_type) [query_engine_type $question $default(ScanSoft,$l,$audio_encoding,engine_type)]]
            set range $channel
            if {$n > 1} {
              set range "${channel}-[expr $channel + $n - 1]"
            }
            lappend config(VVTdefaults) "ttsp.${range}.Tts.engineType: $engine_type"
            lappend config(VVTdefaults) "ttsp.${range}.Tts.languages: [translate_language_code STDVVT $l]"
            lappend config(VVTdefaults) "ttsp.${range}.Tts.oem: Scansoft"
            if {$default(ScanSoft,$l,encoding) == 1} {
              lappend config(VVTdefaults) "ttsp.${range}.ScansoftApi.playPcmCompanding: alaw"
            }
            incr channel $n
          }
        }
      }
      if {$channel >= 1} {
        lappend config(VVTdefaults) "Pm.numberOfTtsServers: $channel"
        set configure(Tts) 1
      }
      incr sc $channel
      if {$configure(Tts) == 0} {
        if {[info exists default(Tts,noprocesses)] == 0} {
          set default(Tts,noprocesses) 0
        }
        if {[set default(Tts,noprocesses) [query_yesno "You have defined no text-to-speech engines, so you won't be able to use this server for text-to-speech. Are you sure" $default(Tts,noprocesses)]] == 1} {
          set noprocesses $max(retry)
        } else {
          incr noprocesses
          if {$noprocesses >= $max(retry)} {
            puts stderr "You have reached the maximum number of retries for this question."
            puts stderr "Continuing without any defined text-to-speech engines."
            puts stderr ""
          }
        }
      }
    }
  }
  if {$configure(Tts) == 1} {
    switch -exact -- $tcl_platform(platform) {
      windows {
        lappend config(VVTdefaults) "ttsp.*.Tts.resetPerUtterance: 0"
      }
    }
  }
  if {$configure(Asr) == 1 || $configure(Tts) == 1} {
# currently the router is only supported on the server-side
# later-on the section below can be used to ask if a router should be started
# on the server or not
#    if {$sc == 0} {
# we have a router only server so do not ask for confirmation
      set configure(router) 1
#    } else {
#      set configure(router) [query_yesno "Do you want to run a router process on this host" 1]
#    }
    if {$configure(router) == 1} {
      set config(VVTdefaults) [linsert $config(VVTdefaults) 0 "Pm.useRouter: 1"]
    } else {
      if {[info exists default(routerHostServer)] == 0} {
        set default(routerHostServer) ""
      }
      set h [set default(routerHostServer) [query_hostname "On which host is running the router process" $default(routerHostServer) 1 1]]
      lappend config(VVTdefaults) "*.*.TsmRouter.routerHost: $h"
      lappend config(VVTdefaults) "*.*.TsmRouter.routersToScan: [llength $h]"
      if {$configure(Asr) == 1} {
        lappend config(VVTdefaults) "*.*.Asr.routerHost: $h"
      }
      if {$configure(Tts) == 1} {
        lappend config(VVTdefaults) "*.*.Tts.routerHost: $h"
      }
    }
  }
  if {[is Client] == 1 &&
      ($configure(Asr) == 1 || $configure(Tts) == 1)} {
# we have a client/server combination check if other servers should be
# contacted beside the localhost
    if {[info exists default(Client,localServer)] == 0} {
      set default(Client,localServer) 0
    }
    if {[set default(Client,localServer) [query_yesno "Do you have other servers you want this client to connect" $default(Client,localServer)]] == 1} {
      if {[info exists default(ClientServer,routerHost)] == 0} {
        set default(ClientServer,routerHost) ""
      }
      set h [set default(ClientServer,routerHost) [query_hostname "On which host(s) is/are running the additional router process(es)" $default(ClientServer,routerHost) 0]]
      set h "localhost $h"
      if {[llength $h] > 1} {
        lappend config(VVTdefaults) "*.*.TsmRouter.routerHost: $h"
        lappend config(VVTdefaults) "*.*.TsmRouter.routersToScan: [llength $h]"
      }
    }
  }
}
# we have a client and in case there is also a server nothing of the server
# has been configured
if {[is Client] == 1 && $configure(Asr) == 0 && $configure(Tts) == 0} {
  if {[info exists default(Client,routerHost)] == 0} {
    set default(Client,routerHost) ""
  }
  set h [set default(Client,routerHost) [query_hostname "On which host(s) is/are running the router process(es) for this client" $default(Client,routerHost) 1]]
  lappend config(VVTdefaults) "*.*.TsmRouter.routerHost: $h"
  lappend config(VVTdefaults) "*.*.TsmRouter.routersToScan: [llength $h]"
}

write_config
write_defaults
Exit
